#!/usr/local/bin/perl
#
# Take a bib file, creates a corresponding .bbl file, 
# convert it to HTML and optionally produce also a .dvi
#
# usage:
#
# dmibib2html [-s style] [-v] [-h] [-l language] [-d dvifile] [-I itemizecommand]
#             [-S htmlseparator] [-L headerlevel] [-o output file]
#             [-H headcvtfile] source-bib-file html-output-file
##
#
require 'getopts.pl';
#
#set default values: to change if you install custom
#
$HOME="/users/formel/dicosmo";
$opt_H="$HOME/TeX/Bibliography/bibabel/headers";
#
#set default values: should not change if you install custom
#
$TMPDIR="/tmp";
$TMP="$TMPDIR/dmibib2html$$";
$opt_s="dmialpha";
$BIBTEX="bibtex";
$HERE=`pwd`;chop $HERE;
$opt_S="";
$opt_L="2";
$opt_I="UL";
$opt_l="french";
###################################################
#
# From bib2html BEGIN
#
###################################################
# Return the command needed to open a (perhaps compressed) file,
# as well as the type of compression.
sub openCommand {
    local($path) = @_;
    local($cmd);
    local($cmp);

command: {
	($path =~ m/\.Z$/ &&
	  ($cmd = "uncompress -c $path |", $cmp = "Compressed", last command));
	($path =~ m/\.g?z$/ &&
	  ($cmd = "gzip -d -c $path |", $cmp = "Gzipped", last command));
	($cmd = "<$path", $cmp = "", last command);
    }

    ($cmd, $cmp);
}

@paperTypes = ("PS", "PDF", "DVI", "ABSTRACT");

sub PSPageCount {
    local($cmd) = @_;
    local($pageCount);

    #print "in PSPageCount $cmd\n";

    open(FILE, $cmd) || (warn "error opening $cmd: $!", return 0);

    local($_);
    local($/) = "\n";

line:
    while (<FILE>) {
	last line if m/^%%EndComments/;
	if (m/^%%Pages:\s*(\d+)/) {
	    $pageCount = $1 if ($1 > 0);
	    last line;
	}
    }
    close(FILE);

    $pageCount;
}

# Don't know how to determine page count of PDF file.
sub PDFPageCount { 0; }

# Don't know how to determine page count of ABSTRACT file.
sub ABSTRACTPageCount { 0; }

sub DVIPageCount {
    local($cmd) = @_;
    local($pageCount) = 0;

    #print "in DVIPageCount $cmd\n";

    if ($cmd =~ m/^</) {
	# Simple file.
	$cmd = "dviselect : $cmd >/dev/null";
    } else {
	# Compressed file.
	$cmd .= "dviselect : >/dev/null";
    }

    # Look at dviselect's stderr.
    open(DVISELECT, "-|") || (open(STDERR, ">&STDOUT"), exec $cmd);

    local($_);
    local($/) = "\n";
line:
    while (<DVISELECT>) {
	if (m/[Ww]rote (\d+) pages/) {
	    $pageCount = $1;
	    last line;
	}
    }
    close(DVISELECT);

    $pageCount;
}

# Make an intelligent link to a paper file.
sub doPaperLinks {
    local($file);

papertype:
    foreach $paper (@paperTypes) {
	$*=1;
	if (($file) = m/<!-- $paper: [\n]*(.+) -->/) {  # was \S+ !!! RDC
            $file =~s/\s//g;
            print STDERR $file,"\n";
	    local($path) = $htmldir . $file;

#	    if (! -f $path) {
#		print STDERR "couldn't find $path\n";
#		next papertype;
#	    }

	    local($opencmd);
	    local($size);
	    local($sstr);
	    local($pageCountRoutine);
	    local($pageCount);
	    local($pstr) = "";

	    ($opencmd, $cstr) = &openCommand($path);

	    # Get size.
	    $size = -s _;
	    $sstr = ", $size bytes";

	    # Get page count.
	    $pageCountRoutine = $paper . "PageCount";
	    $pageCount = &$pageCountRoutine($opencmd);
	    $pstr = ", $pageCount pages" if ($pageCount > 0);

	    # Get compression type.
	    $cstr = "$cstr " if ($cstr ne "");

#	    print STDERR "found $paper $file$pstr$sstr\n";
	    s/<!-- $paper: [\n]*.+ -->/(<A HREF=\"$file\">${cstr}$paper<\/A>)/;      # was \S+ !!! RDC
	}
    }
}

###################################################
#
# From bib2html END
#
###################################################
do Getopts('s:l:d:o:S:L:H:I:vh');

if (defined($opt_h)) {system("sed  -n -e '2,/##/p' `which $0`"); exit 0;};

if ($opt_l eq "french") {$LATEXOPTS="frenchart,";};

$FILENAME=$ARGV[0];
$FILENAME=`basename $FILENAME .bib`;chop $FILENAME;
$OUTDIR="$TMPDIR/$FILENAME";
if ($#ARGV == 1){$opt_o="$HERE/$ARGV[1]"; $THEHTMLFILE=$ARGV[1];}
elsif (!defined($opt_o)) {$opt_o="$HERE/$FILENAME.html"; $THEHTMLFILE="$FILENAME.html";};
#
# Copy WWW bst files to TMPDIR
#
system("cp /users/formel/dicosmo/TeX/Bibliography/BST/WWW/*.bst $TMPDIR");
#
# Copy FILENAME to TMPDIR
#
system("cp $FILENAME.bib $TMPDIR");
chdir($TMPDIR);
#
# create fake aux file
#
open (OUT, "> $TMP.aux");
print OUT <<EOF;
\\citation{*}
\\bibdata{$FILENAME}
\\bibstyle{$opt_s}
EOF
close(OUT);

#
# call bibtex
#
if (defined($opt_v)) 
     {$status=system("$BIBTEX $TMP > $TMP.blog 2>&1");}
else {$status=system("$BIBTEX $TMP");};

#
# modify the file...
#
#
# Load headers into assoc array fr the chosen language
#
open(HEADERS, "$opt_H");
while (<HEADERS>) {
                # skip comments and empty lines
                   next if /^#/;
                   next if (!/:/);
                   chop; $lang=$_; $headabbrv=$_; 
                #parse line into lang:headabbrv:headfull
                   $headabbrv=~s/[^:]*://; $headfull=$headabbrv;
                   $headabbrv=~s/:.*//; $headfull=~s/.*://; $headfull=~s/ *$//; 
                   $lang=~s/:.*//; $lang=~s/ *//; $lang{$lang}++;
                # now fill in the array
                   if ($lang eq $opt_l) {$header{$headabbrv}=$headfull;};
               };
close(HEADERS);

#multi-line matching
$*=1;

#
# handle horrible Bibtex cuts
#
        open(OUT, "> $TMP.pl");
        print OUT <<EOF;
\$*=1;
s/%\\n//; 
s/(<!-- [A-Z]*:.*)[ \\n]*/\\1 /;   # CHANGED 18/3/96: was s/(<!-- [A-Z]*:)[ \\n]*/\\1 /;
EOF
        close(OUT);
open (IN, "/usr/local/bin/perl -p $TMP.pl < $TMP.bbl |");
open(OUT, "| tex-to-html >$opt_o");
while(<IN>){
    if (/BibHeadline/) {
        # modify the line as necessary
        $abbrv=$_; $abbrv=~s/.*BibHeadline: *([^ ]*) *.*/\1/; chop $abbrv;
        s/BibHeadline: *([^ ]*) *!/BibHeadline> $header{$abbrv}<!/;
        s%<!BibHeadline>%$opt_S <H$opt_L>%;
        s%<!/BibHeadline>%</H$opt_L>%;
    };
    #
    # get rid of htmladdnormallink stuff: DOES NOT WORK PROPERLY ON MULTILINE!!!
    #
    s/\\htmladdnormallink{[^}]*}{(([^}]|[\n])*)}/\1/;
    # Finally, take care of cross-references (this is best done via perl and associative arrays!) 
    #
    s%[~]*\\cite{([^}]*)}%<A HREF="#\1">this</A>%;
    #
    # And use proper Listing formatting: (dependency with btxbst.doc)
    #
    s%PUBLIST>%$opt_I>%;   
 
    &doPaperLinks;

    #
    # Handle authors with split-names
    #
    if (/<!-- Authors:.*Di Cosmo/) {s/Di Cosmo/Di Cosmo Dicosmo/};
    print OUT $_;
};                              
close(OUT);
close(IN);

#
# If the dvi file has been asked for, create it!
#
if (defined($opt_d)) {
        #
        # remove WWW bst files
        #
        unlink <$TMPDIR/*.bst>;
        #
        # Copy TeX-WWW bst files to TMPDIR
        #
        system("cp /users/formel/dicosmo/TeX/Bibliography/BST/*.bst $TMPDIR");
        #
        # create fake tex file
        #
        open(OUT, "> $FILENAME.tex");
        print OUT <<EOF;
        \\documentstyle[a4, $LATEXOPTS html]{article}
        \\begin{document}
        \\nocite{*}
        \\bibliographystyle{$opt_s}
        \\bibliography{$FILENAME}
        \\end{document}
EOF
        close(OUT);
        #
        print "Building dvi file $opt_d...";
        system("TEXINPUTS=\$HOME/TeX/Bibliography/bibabel':'\$TEXINPUTS; export TEXINPUTS; \
                latex $FILENAME;bibtex $FILENAME; \
                (sed 's/dmi.default.headlines/dmi.$opt_l.headlines/' < $FILENAME.bbl > $FILENAME.bbl.l);\
                mv $FILENAME.bbl.l $FILENAME.bbl \
                latex $FILENAME;latex $FILENAME ; cp $FILENAME.dvi $HERE/$opt_d");
        print "Building dvi file $opt_d...Done.";
        };
#
# Clean up
#
print "Cleaning up ...\n";
unlink "$TMP.aux", "$TMP.bbl" , "$TMP.blg" , "$TMP.blog" , "$TMP.bib", "$TMP.pl";
print "Cleaning up ...Done.\n";



